@@ -88,12 +88,14 @@ InitCandidateWindow(MUIMSessionHandle inHandle)
   require(appBundle, CantFindMainBundle);
     
   //baseURL = CFBundleCopyPrivateFrameworksURL(appBundle);
-  baseURL = CFURLCreateWithString(NULL,
-                                  CFSTR("/Library/Components/MacUIM.component/Contents/Frameworks/"),
-                                  NULL);
-  baseURL2 = CFURLCreateWithString(NULL,
-                                   CFSTR("/Library/Frameworks/"),
-                                   NULL);
+  baseURL =
+    CFURLCreateWithString(NULL,
+                          CFSTR("/Library/Components/MacUIM.component/Contents/Frameworks/"),
+                          NULL);
+  baseURL2 =
+    CFURLCreateWithString(NULL,
+                          CFSTR("/Library/Frameworks/"),
+                          NULL);
   require(baseURL, CantCopyURL);
   require(baseURL2, CantCopyURL);
 
@@ -150,6 +152,11 @@ ShowCandidateWindow(MUIMSessionHandle inHandle,
   OSStatus res = noErr;
   UInt32 i;
   OSStatus (*showPtr)(SInt16, SInt16);
+  OSStatus (*pagePtr)(int, int);
+  //char idx[16];
+
+  if (!(*inHandle)->fBundleRef)
+    return noErr;
 
   //if ((*inHandle)->fWindowOpened)
   //  return noErr;
@@ -211,6 +218,24 @@ ShowCandidateWindow(MUIMSessionHandle inHandle,
     if (selectPtr)
       res = (*selectPtr)((*inHandle)->fCandidateIndex -
                          (*inHandle)->fLayoutBegin);
+
+    pagePtr = CFBundleGetFunctionPointerForName((*inHandle)->fBundleRef,
+                                                CFSTR("setPage"));
+    if (pagePtr)
+      res = (*pagePtr)((*inHandle)->fCandidateIndex + 1,
+                       (*inHandle)->fNRCandidates);
+  }
+  else {
+    OSStatus (*deselectPtr)();
+    deselectPtr = CFBundleGetFunctionPointerForName((*inHandle)->fBundleRef,
+                                                  CFSTR("deselectCandidate"));
+    if (deselectPtr)
+      res = (*deselectPtr)();
+
+    pagePtr = CFBundleGetFunctionPointerForName((*inHandle)->fBundleRef,
+                                                CFSTR("setPage"));
+    if (pagePtr)
+      res = (*pagePtr)(0, (*inHandle)->fNRCandidates);
   }
 
   // call function to show window
@@ -230,6 +255,9 @@ HideCandidateWindow(MUIMSessionHandle inHandle)
   OSStatus res = noErr;
   OSStatus (*hidePtr)();
 
+  if (!(*inHandle)->fBundleRef)
+    return noErr;
+
   //if (!(*inHandle)->fWindowOpened)
   //  return noErr;
 
