@@ -44,6 +44,7 @@
 #include "MUIMScript.h"
 #include "UIMCallback.h"
 #include "CandidateCarbon.h"
+#include "KeycodeToUKey.h"
 
 
 const short kMENU_Pencil = kBaseResourceID + 1;
@@ -281,6 +282,7 @@ MUIMSessionOpen(ComponentInstance inComponentInstance,
     (**outSessionHandle)->fFixBuffer = NULL;
     (**outSessionHandle)->fFixLen = 0;
     (**outSessionHandle)->fOldPreedit = NULL;
+    (**outSessionHandle)->fOldPreeditLen = 0;
 
     (**outSessionHandle)->fBundleRef = NULL;
 
@@ -319,7 +321,8 @@ MUIMSessionActivate(MUIMSessionHandle inSessionHandle)
 
   gActiveSession = inSessionHandle;
 
-  long keyboardID = GetScriptVariable(GetScriptManagerVariable(smLastScript), smScriptKeys);
+  long keyboardID = GetScriptVariable(GetScriptManagerVariable(smLastScript),
+                                      smScriptKeys);
   SetScriptVariable (smJapanese, smScriptKeys, keyboardID);
 
   DEBUG_PRINT("MUIMSessionActivate() gActiveSession=%p\n",
@@ -383,7 +386,7 @@ MUIMSessionEvent(MUIMSessionHandle inSessionHandle, EventRef inEventRef)
     GetEventParameter(inEventRef, kEventParamKeyModifiers, typeUInt32, nil,
                       sizeof(modifiers), nil, &modifiers);
 
-    DEBUG_PRINT("MUIMSessionEvent() keycode=%lx, char=%c, charCode=%x, modifiers=%lx\n",
+    DEBUG_PRINT("MUIMSessionEvent() keycode=0x%lx, char=%c, charCode=0x%x, modifiers=0x%lx\n",
                 keyCode, charCode, charCode, modifiers);
 
     handled = MUIMHandleInput(inSessionHandle, keyCode, charCode, modifiers);
@@ -415,56 +418,36 @@ MUIMGetActiveSession(void)
 }
 
 Boolean
-MUIMHandleInput(MUIMSessionHandle inSessionHandle, UInt32 inKeycode, unsigned char inCharCode, UInt32 inModifiers)
+MUIMHandleInput(MUIMSessionHandle inSessionHandle, UInt32 inKeycode,
+                unsigned char inCharCode, UInt32 inModifiers)
 {
   Boolean handled;
+  Boolean isScriptKey;
   int key = 0, mod = 0;
   int rv;
+  int i;
 
   handled = FALSE;
+  isScriptKey = FALSE;
 
-  DEBUG_PRINT("MUIMHandleInput() inCharCode=%02x inModifiers=%lx\n", inCharCode, inModifiers);
+  DEBUG_PRINT("MUIMHandleInput() inCharCode=%02x inModifiers=%lx\n",
+              inCharCode, inModifiers);
 
-  switch (inKeycode) {
-  case 0x24:
-  case 0x34:
-    key = UKey_Return;
-    break;
-  case 0x33:
-    key = UKey_Backspace;
-    break;
-  case 0x35:
-    key = UKey_Escape;
-    break;
-  case 0x73:
-    key = UKey_Home;
-    break;
-  case 0x74:
-    key = UKey_Prior;
-    break;
-  case 0x77:
-    key = UKey_End;
-    break;
-  case 0x79:
-    key = UKey_Next;
-    break;
-  case 0x7b:
-    key = UKey_Left;
-    break;
-  case 0x7c:
-    key = UKey_Right;
-    break;
-  case 0x7d:
-    key = UKey_Down;
-    break;
-  case 0x7e:
-    key = UKey_Up;
-    break;
-  default:
-    key = ConvertKeyVal(inCharCode);
-    mod = ConvertModifier(inModifiers);
-    break;
+  /* Check for special keys first */
+  for (i = 0; KeycodeToUKey[i].ukey; i++) {
+    if (KeycodeToUKey[i].keycode == inKeycode) {
+      key = KeycodeToUKey[i].ukey;
+      break;
+    }
   }
+  if (key == UKey_Private1 || key == UKey_Private2)
+    isScriptKey = TRUE;
+  
+  /* Then convert normal keys */
+  if (key == 0)
+    key = ConvertKeyVal(inCharCode);
+
+  mod = ConvertModifier(inModifiers);
 
   DEBUG_PRINT("MUIMHandleInput() key=0x%x mod=0x%x\n", key, mod);
 
@@ -473,15 +456,16 @@ MUIMHandleInput(MUIMSessionHandle inSessionHandle, UInt32 inKeycode, unsigned ch
 
   uim_release_key((*inSessionHandle)->fUC, key, mod);
 
-  if (!rv) {
+  if (!rv || isScriptKey) {
     handled = TRUE;
-
+    /*
     if ((*inSessionHandle)->fFixLen > 0) {
       MUIMUpdateActiveInputArea(inSessionHandle, TRUE);
       free((*inSessionHandle)->fFixBuffer);
       (*inSessionHandle)->fFixBuffer = NULL;
       (*inSessionHandle)->fFixLen = 0;
     }
+    */
   }
 
   return handled;
@@ -558,14 +542,6 @@ MUIMPencilMenuEventHandler(EventHandlerCallRef inEventHandlerCallRef,
   return result;
 }
 
-void
-MUIMLog(unsigned char *message)
-{
-/*
-  fprintf( stderr, message );
-*/
-}
-
 static int
 ConvertKeyVal(int inKey)
 {
@@ -579,12 +555,14 @@ ConvertKeyVal(int inKey)
 static int
 ConvertModifier(int inMod)
 {
+  int modifier = 0;
+
   DEBUG_PRINT("ConvertModifier() inMod=%lx\n", inMod);
 
-  if (inMod == 0x200)
-    return UMod_Shift;
-  if (inMod == 0x1000)
-    return UMod_Control;
+  if (inMod & 0x200)
+    modifier += UMod_Shift;
+  if (inMod & 0x1000)
+    modifier += UMod_Control;
 
-  return 0;
+  return modifier;
 }
