@@ -62,6 +62,7 @@ typedef struct _MUIMSessionRecord
   UInt32 fFixLen;
 
   UniCharPtr fOldPreedit;
+  UInt32 fOldPreeditLen;
 
   SInt32 fNRCandidates;
   SInt32 fCandidateIndex;
@@ -122,9 +123,6 @@ MUIMGetActiveSession(void);
 Boolean
 MUIMHandleInput(MUIMSessionHandle inSessionHandle, UInt32 inKeycode, unsigned char inCharCode, UInt32 inModifiers);
 
-void
-MUIMLog(unsigned char *message);
-
 void
 GetPreeditSegment(PreeditSegment *inSegment, UniCharPtr *outStr,
                   UInt32 *outLen);
