@@ -70,6 +70,13 @@ UIMCommitString(void *ptr, char *str)
     DumpString("utf16", (char *) (*handle)->fFixBuffer,
     (*handle)->fFixLen * sizeof(UniChar));
   */
+
+  if ((*handle)->fFixLen > 0) {
+    MUIMUpdateActiveInputArea(handle, TRUE);
+    free((*handle)->fFixBuffer);
+    (*handle)->fFixBuffer = NULL;
+    (*handle)->fFixLen = 0;
+  }
 }
 
 void
@@ -109,24 +116,28 @@ UIMPreeditPushback(void *ptr, int attr, char *str)
 
 #ifdef DEBUG
   {
-    char *attrstr = "";
+    char *tmpstr = "";
+    char attrstr[50];
 
-    switch (attr) {
-    case UPreeditAttr_None:
-      attrstr = "None";
-      break;
-    case UPreeditAttr_UnderLine:
-      attrstr = "UnderLine";
-      break;
-    case UPreeditAttr_Reverse:
-      attrstr = "Reverse";
-      break;
-    case UPreeditAttr_Cursor:
-      attrstr = "Cursor";
-      break;
-    case UPreeditAttr_Separator:
-      attrstr = "Separator";
-      break;
+    if (attr & UPreeditAttr_None) {
+      tmpstr = "None";
+      sprintf(attrstr, "%s", tmpstr);
+    }
+    if (attr & UPreeditAttr_UnderLine) {
+      tmpstr = "UnderLine";
+      sprintf(attrstr, "%s %s", attrstr, tmpstr);
+    }
+    if (attr & UPreeditAttr_Reverse) {
+      tmpstr = "Reverse";
+      sprintf(attrstr, "%s %s", attrstr, tmpstr);
+    }
+    if (attr & UPreeditAttr_Cursor) {
+      tmpstr = "Cursor";
+      sprintf(attrstr, "%s %s", attrstr, tmpstr);
+    }
+    if (attr & UPreeditAttr_Separator) {
+      tmpstr = "Separator";
+      sprintf(attrstr, "%s %s", attrstr, tmpstr);
     }
 
     DEBUG_PRINT("UIMPreeditPushback() fSegmentCount=%lu attr=%s len=%lu\n",
@@ -302,10 +313,10 @@ GetPreeditString(MUIMSessionHandle inHandle, UniCharPtr *outStr,
   DEBUG_PRINT("GetPreeditString() fSegmentCount=%lu\n",
               (*inHandle)->fSegmentCount);
   for (i = 0; i < (*inHandle)->fSegmentCount; i++) {
-    if ((*inHandle)->fSegments[i].fAttr & UPeAttr_Cursor) {
+    GetPreeditSegment(&((*inHandle)->fSegments[i]), &str, &len);
+    if ((*inHandle)->fSegments[i].fAttr & UPreeditAttr_Cursor) {
       pos = len;
     }
-    GetPreeditSegment(&((*inHandle)->fSegments[i]), &str, &len);
     DEBUG_PRINT("GetPreeditString() i=%lu len=%lu\n", i, len);
   }
   DEBUG_PRINT("GetPreeditString() len=%lu pos=%lu\n", len, pos);
